package be.cylab.java.wowa.training;

import java.util.logging.Logger;

public class TrainerParameters {
    public static final int SELECTION_METHOD_RWS = 374;
    public static final int SELECTION_METHOD_TOS = 942;


    private Logger logger;
    private int populationSize;
    private int crossoverRate;
    private int mutationRate;
    private int selectionMethod;

    private int maxGenerationNumber;
    private float triggerDistance;


    /**
     * Contains parameters for the WOWA trainer.
     *
     * @param population_size Size of the population for the Genetic Algorithm (greater than 2)
     * @param crossover_rate Crossover rate for the genetic algorithm. Percentage of the population kept to generate the next generation (1-99)
     * @param mutation_rate Mutation rate for the genetic algorithm. percentage of genes mutated in the next generation (1-99)
     * @param selection_method Method to select parents for the next generation. RWS or TOS
     * @param max_generation_number Max generation number before the algorithm stop
     */
    public TrainerParameters(
            //Logger logger,
            int population_size,
            int crossover_rate,
            int mutation_rate,
            int selection_method,
            int max_generation_number

    ) {
        //this.setLogger(logger);
        this.setPopulationSize(population_size);
        this.setCrossoverRate(crossover_rate);
        this.setMutationRate(mutation_rate);
        this.setSelectionMethod(selection_method);
        this.setMaxGenerationNumber(max_generation_number);

    }

    /**
     *
     * @return int, number of parents. Depends on the crossover rate
     */
    public int getNumberParents() {
        int nbreParents = Math.round(this.getPopulationSize() * (1 - this.getCrossoverRate() / 100));
        if (nbreParents % 2 == 1) {
            nbreParents++;
        }
        return nbreParents;
    }

    /**
     *
     * @return Logger
     */
    public Logger getLogger() {
        return logger;
    }


    public int getPopulationSize() {
        return populationSize;
    }

    private void setPopulationSize(int populationSize) throws ArithmeticException {
        if (populationSize < 2) {
            throw new ArithmeticException("Population must be higher than 2");
        }
        this.populationSize = populationSize;
    }

    private int getCrossoverRate() {
        return crossoverRate;
    }

    private void setCrossoverRate(int crossoverRate) {
        this.crossoverRate = crossoverRate;
    }

    public int getMutationRate() {
        return mutationRate;
    }

    private void setMutationRate(int mutationRate) {
        this.mutationRate = mutationRate;
    }

    public int getSelectionMethod() {
        return selectionMethod;
    }

    private void setSelectionMethod(int selectionMethod) {
        this.selectionMethod = selectionMethod;
    }


    public int getMaxGenerationNumber() {
        return maxGenerationNumber;
    }

    private void setMaxGenerationNumber(int maxGenerationNumber) {
        this.maxGenerationNumber = maxGenerationNumber;
    }

    public float getTriggerDistance() {
        return triggerDistance;
    }

    public void setTriggerDistance(float triggerDistance) {
        this.triggerDistance = triggerDistance;
    }
}
